import { EntityManager, EntitySubscriberInterface, InsertEvent, RemoveEvent, UpdateEvent } from "typeorm";
export interface HistoryEntitySubscriberInterface<EntityType, HistoryEntityType> extends EntitySubscriberInterface<EntityType> {
    entity: Function;
    historyEntity: Function;
    createHistoryEntity(manager: EntityManager, entity: EntityType): HistoryEntityType | Promise<HistoryEntityType>;
    beforeInsertHistory(history: HistoryEntityType, entity: Readonly<EntityType>): HistoryEntityType | Promise<HistoryEntityType>;
    afterInsertHistory(history: HistoryEntityType, entity: Readonly<EntityType>): void | Promise<void>;
    beforeUpdateHistory(history: HistoryEntityType, entity: Readonly<EntityType>): HistoryEntityType | Promise<HistoryEntityType>;
    afterUpdateHistory(history: HistoryEntityType, entity: Readonly<EntityType>): void | Promise<void>;
    beforeRemoveHistory(history: HistoryEntityType, entity: Readonly<EntityType>): HistoryEntityType | Promise<HistoryEntityType>;
    afterRemoveHistory(history: HistoryEntityType, entity: Readonly<EntityType>): void | Promise<void>;
}
export declare abstract class HistoryEntitySubscriber<EntityType, HistoryEntityType> implements HistoryEntitySubscriberInterface<EntityType, HistoryEntityType> {
    beforeInsertHistory(history: HistoryEntityType, entity: Readonly<EntityType>): HistoryEntityType | Promise<HistoryEntityType>;
    afterInsertHistory(history: HistoryEntityType, entity: Readonly<EntityType>): void | Promise<void>;
    beforeUpdateHistory(history: HistoryEntityType, entity: Readonly<EntityType>): HistoryEntityType | Promise<HistoryEntityType>;
    afterUpdateHistory(history: HistoryEntityType, entity: Readonly<EntityType>): void | Promise<void>;
    beforeRemoveHistory(history: HistoryEntityType, entity: Readonly<EntityType>): HistoryEntityType | Promise<HistoryEntityType>;
    afterRemoveHistory(history: HistoryEntityType, entity: Readonly<EntityType>): void | Promise<void>;
    abstract get entity(): Function;
    abstract get historyEntity(): Function;
    listenTo(): Function;
    createHistoryEntity(manager: Readonly<EntityManager>, entity: EntityType): HistoryEntityType;
    afterInsert(event: InsertEvent<EntityType>): Promise<void>;
    afterUpdate(event: UpdateEvent<EntityType>): Promise<void>;
    beforeRemove(event: RemoveEvent<EntityType>): Promise<void>;
    private createHistory;
}
