"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HistoryEntitySubscriber = void 0;
const constants_1 = require("./constants");
const history_action_enum_1 = require("./history-action.enum");
class HistoryEntitySubscriber {
    beforeInsertHistory(history, entity) {
        return history;
    }
    afterInsertHistory(history, entity) { }
    beforeUpdateHistory(history, entity) {
        return history;
    }
    afterUpdateHistory(history, entity) { }
    beforeRemoveHistory(history, entity) {
        return history;
    }
    afterRemoveHistory(history, entity) { }
    listenTo() {
        return this.entity;
    }
    createHistoryEntity(manager, entity) {
        return manager.create(this.historyEntity, entity);
    }
    async afterInsert(event) {
        await this.createHistory(event.manager, event.metadata, this.beforeInsertHistory, this.afterInsertHistory, history_action_enum_1.HistoryActionType.CREATED, event.entity);
    }
    async afterUpdate(event) {
        await this.createHistory(event.manager, event.metadata, this.beforeUpdateHistory, this.afterUpdateHistory, history_action_enum_1.HistoryActionType.UPDATED, event.entity);
    }
    async beforeRemove(event) {
        await this.createHistory(event.manager, event.metadata, this.beforeRemoveHistory, this.afterRemoveHistory, history_action_enum_1.HistoryActionType.DELETED, event.entity);
    }
    async createHistory(manager, metadata, beforeHistoryFunction, afterHistoryFunction, action, entity) {
        if (!entity) {
            return;
        }
        const hasActionColumn = Reflect.hasMetadata(constants_1.TYPEORM_HELPER_HISTORY_ACTION_TYPE, entity);
        if (!entity || hasActionColumn) {
            return;
        }
        const history = (await this.createHistoryEntity(manager, entity));
        const actionPropertyName = Reflect.getMetadata(constants_1.TYPEORM_HELPER_HISTORY_ACTION_TYPE, history);
        if (!actionPropertyName) {
            throw new Error(`${this.historyEntity.name} does not have @HistoryActionColumn defined.`);
        }
        Reflect.set(history, actionPropertyName, action);
        let originalIdPropertyName = Reflect.getMetadata(constants_1.TYPEORM_HELPER_HISTORY_ORIGINAL_ID, history);
        {
            // for backward compatibility
            if (!originalIdPropertyName) {
                if (metadata.propertiesMap["originalID"]) {
                    throw new Error(`The originalID has already been defined for ${this.entity.name}. An entity cannot have a property with the same name.`);
                }
                const historyMetadata = manager.connection.getMetadata(this.historyEntity);
                if (historyMetadata?.propertiesMap["originalID"]) {
                    originalIdPropertyName = "originalID";
                }
            }
        }
        if (!originalIdPropertyName) {
            throw new Error(`${this.historyEntity.name} does not have @HistoryOriginalIdColumn defined.`);
        }
        for (const primaryColumn of metadata.primaryColumns) {
            const originalID = Reflect.get(history, primaryColumn.propertyName);
            Reflect.set(history, originalIdPropertyName, originalID);
            Reflect.deleteProperty(history, primaryColumn.propertyName);
        }
        await beforeHistoryFunction(history, entity);
        await manager.save(history);
        await afterHistoryFunction(history, entity);
    }
}
exports.HistoryEntitySubscriber = HistoryEntitySubscriber;
